@@ -21,11 +21,11 @@
  */
 #include "skgbookmarkplugindockwidget.h"
 
-#include <KMenu>
-#include <KIconDialog>
+#include <kicondialog.h>
 
-#include <QHeaderView>
-#include <QResizeEvent>
+#include <qheaderview.h>
+#include <qmenu.h>
+#include <qevent.h>
 
 #include "skgbookmark_settings.h"
 #include "skgtraces.h"
@@ -39,9 +39,9 @@
 
 bool SKGBookmarkPluginDockWidget::m_middleClick = false;
 
-SKGBookmarkPluginDockWidget::SKGBookmarkPluginDockWidget(SKGDocument* iDocument)
-    : SKGWidget(iDocument), m_mainMenu(NULL), m_actDelete(NULL), m_actRename(NULL), m_actChangeIcon(NULL), m_actAddBookmark(NULL), m_actAddBookmarks(NULL),
-      m_actAddBookmarkGroup(NULL), m_actSetAutostart(NULL), m_actUnsetAutostart(NULL)
+SKGBookmarkPluginDockWidget::SKGBookmarkPluginDockWidget(QWidget* iParent, SKGDocument* iDocument)
+    : SKGWidget(iParent, iDocument), m_mainMenu(nullptr), m_actDelete(nullptr), m_actRename(nullptr), m_actChangeIcon(nullptr), m_actAddBookmark(nullptr), m_actAddBookmarks(nullptr),
+      m_actAddBookmarkGroup(nullptr), m_actSetAutostart(nullptr), m_actUnsetAutostart(nullptr), m_modelview(nullptr)
 {
     SKGTRACEINFUNC(1);
     if (!iDocument) {
@@ -57,28 +57,27 @@ SKGBookmarkPluginDockWidget::SKGBookmarkPluginDockWidget(SKGDocument* iDocument)
     ui.kBookmarksList->setAnimated(false);  // Not compatible with double click
     ui.kBookmarksList->setEditTriggers(QAbstractItemView::NoEditTriggers);
 
-    ui.kTools->setIcon(KIcon("configure"));
-    QTimer::singleShot(200, this, SLOT(initMenu()));
+    ui.kTools->setIcon(SKGServices::fromTheme(QStringLiteral("configure")));
 
     ui.kBookmarksList->setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(ui.kBookmarksList, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(showMenu(QPoint)));
+    connect(ui.kBookmarksList, &SKGTreeView::customContextMenuRequested, this, &SKGBookmarkPluginDockWidget::showMenu);
 
     // Add model
-    m_modelview = new SKGObjectModelBase(getDocument(), "v_node", "1=1 ORDER BY f_sortorder, t_name", this, "rd_node_id");
-    SKGSortFilterProxyModel* modelproxy = new SKGSortFilterProxyModel(this);
+    m_modelview = new SKGObjectModelBase(getDocument(), QStringLiteral("v_node"), QStringLiteral("1=1 ORDER BY f_sortorder, t_name"), this, QStringLiteral("rd_node_id"));
+    auto modelproxy = new SKGSortFilterProxyModel(this);
     modelproxy->setSourceModel(m_modelview);
     ui.kBookmarksList->setModel(modelproxy);
 
     ui.kFilterEdit->setProxy(modelproxy);
 
-    connect(ui.kBookmarksList, SIGNAL(selectionChangedDelayed()), this, SLOT(refresh()));
-    connect(ui.kBookmarksList, SIGNAL(pressed(QModelIndex)), this, SLOT(onBeforeOpenBookmark()));
-    connect(ui.kBookmarksList, SIGNAL(clicked(QModelIndex)), this, SLOT(onOpenBookmark(QModelIndex)));
-    connect(ui.kBookmarksList, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(onOpenBookmarkFolder(QModelIndex)));
-    connect(SKGMainPanel::getMainPanel(), SIGNAL(currentPageChanged()), this, SLOT(onPageChanged()), Qt::QueuedConnection);
+    connect(ui.kBookmarksList, &SKGTreeView::selectionChangedDelayed, this, &SKGBookmarkPluginDockWidget::refresh);
+    connect(ui.kBookmarksList, &SKGTreeView::pressed, this, &SKGBookmarkPluginDockWidget::onBeforeOpenBookmark);
+    connect(ui.kBookmarksList, &SKGTreeView::pressed, this, &SKGBookmarkPluginDockWidget::onOpenBookmark);
+    connect(ui.kBookmarksList, &SKGTreeView::doubleClicked, this, &SKGBookmarkPluginDockWidget::onOpenBookmarkFolder);
+    connect(SKGMainPanel::getMainPanel(), &SKGMainPanel::currentPageChanged, this, &SKGBookmarkPluginDockWidget::onPageChanged);
 
-    connect(m_modelview, SIGNAL(beforeReset()), ui.kBookmarksList, SLOT(saveSelection()));
-    connect(m_modelview, SIGNAL(afterReset()), ui.kBookmarksList, SLOT(resetSelection()));
+    connect(m_modelview, &SKGObjectModelBase::beforeReset, ui.kBookmarksList, &SKGTreeView::saveSelection);
+    connect(m_modelview, &SKGObjectModelBase::afterReset, ui.kBookmarksList, &SKGTreeView::resetSelection);
 
     ui.kBookmarksList->setTextResizable(false);
 
@@ -89,16 +88,16 @@ SKGBookmarkPluginDockWidget::SKGBookmarkPluginDockWidget(SKGDocument* iDocument)
 SKGBookmarkPluginDockWidget::~SKGBookmarkPluginDockWidget()
 {
     SKGTRACEINFUNC(1);
-    m_mainMenu = NULL;
-    m_actDelete = NULL;
-    m_actRename = NULL;
-    m_actChangeIcon = NULL;
-    m_actAddBookmark = NULL;
-    m_actAddBookmarks = NULL;
-    m_actAddBookmarkGroup = NULL;
-    m_actSetAutostart = NULL;
-    m_actUnsetAutostart = NULL;
-    m_modelview = NULL;
+    m_mainMenu = nullptr;
+    m_actDelete = nullptr;
+    m_actRename = nullptr;
+    m_actChangeIcon = nullptr;
+    m_actAddBookmark = nullptr;
+    m_actAddBookmarks = nullptr;
+    m_actAddBookmarkGroup = nullptr;
+    m_actSetAutostart = nullptr;
+    m_actUnsetAutostart = nullptr;
+    m_modelview = nullptr;
 }
 
 QWidget* SKGBookmarkPluginDockWidget::mainWidget()
@@ -110,52 +109,53 @@ void SKGBookmarkPluginDockWidget::initMenu()
 {
     if (!m_mainMenu) {
         // Build contextual menu
-        m_mainMenu = new KMenu(ui.kBookmarksList);
-        QAction* actExpandAll = m_mainMenu->addAction(KIcon("format-indent-more"), i18nc("Noun, user action", "Expand all"));
-        connect(actExpandAll, SIGNAL(triggered(bool)), ui.kBookmarksList, SLOT(expandAll()));
+        m_mainMenu = new QMenu(ui.kBookmarksList);
+        QAction* actExpandAll = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("format-indent-more")), i18nc("Noun, user action", "Expand all"));
+        connect(actExpandAll, &QAction::triggered, ui.kBookmarksList, &SKGTreeView::expandAll);
 
-        QAction* actCollapseAll = m_mainMenu->addAction(KIcon("format-indent-less"), i18nc("Noun, user action", "Collapse all"));
-        connect(actCollapseAll, SIGNAL(triggered(bool)), ui.kBookmarksList, SLOT(collapseAll()));
-        ui.kBookmarksList->insertAction(0, actCollapseAll);
+        QAction* actCollapseAll = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("format-indent-less")), i18nc("Noun, user action", "Collapse all"));
+        connect(actCollapseAll, &QAction::triggered, ui.kBookmarksList, &SKGTreeView::collapseAll);
+        ui.kBookmarksList->insertAction(nullptr, actCollapseAll);
 
         m_mainMenu->addSeparator();
 
-        m_actAddBookmark = m_mainMenu->addAction(KIcon("list-add"), i18nc("Verb", "Bookmark current page"));
-        connect(m_actAddBookmark, SIGNAL(triggered(bool)), this, SLOT(onAddBookmark()));
+        m_actAddBookmark = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("list-add")), i18nc("Verb", "Bookmark current page"));
+        connect(m_actAddBookmark, &QAction::triggered, this, &SKGBookmarkPluginDockWidget::onAddBookmark);
 
-        m_actAddBookmarks = m_mainMenu->addAction(KIcon("list-add"), i18nc("Verb", "Bookmark all pages"));
-        connect(m_actAddBookmarks, SIGNAL(triggered(bool)), this, SLOT(onAddBookmarks()));
+        m_actAddBookmarks = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("list-add")), i18nc("Verb", "Bookmark all pages"));
+        connect(m_actAddBookmarks, &QAction::triggered, this, &SKGBookmarkPluginDockWidget::onAddBookmarks);
 
-        m_actAddBookmarkGroup = m_mainMenu->addAction(KIcon("folder-new"), i18nc("Verb", "Add bookmark group"));
-        connect(m_actAddBookmarkGroup, SIGNAL(triggered(bool)), this, SLOT(onAddBookmarkGroup()));
+        m_actAddBookmarkGroup = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("folder-new")), i18nc("Verb", "Add bookmark group"));
+        connect(m_actAddBookmarkGroup, &QAction::triggered, this, &SKGBookmarkPluginDockWidget::onAddBookmarkGroup);
 
         m_mainMenu->addSeparator();
 
-        m_actDelete = m_mainMenu->addAction(KIcon("edit-delete"), i18nc("Verb, delete an item", "Delete"));
-        connect(m_actDelete, SIGNAL(triggered(bool)), this, SLOT(onRemoveBookmark()));
+        m_actDelete = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("edit-delete")), i18nc("Verb, delete an item", "Delete"));
+        connect(m_actDelete, &QAction::triggered, this, &SKGBookmarkPluginDockWidget::onRemoveBookmark);
 
         m_mainMenu->addSeparator();
 
-        m_actSetAutostart = m_mainMenu->addAction(KIcon("user-online"), i18nc("Verb, automatically load when the application is started", "Autostart"));
-        connect(m_actSetAutostart, SIGNAL(triggered(bool)), this, SLOT(onSetAutostart()));
+        m_actSetAutostart = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("media-playback-start")), i18nc("Verb, automatically load when the application is started", "Autostart"));
+        connect(m_actSetAutostart, &QAction::triggered, this, &SKGBookmarkPluginDockWidget::onSetAutostart);
 
-        m_actUnsetAutostart = m_mainMenu->addAction(KIcon("user-offline"), i18nc("Verb", "Remove Autostart"));
-        connect(m_actUnsetAutostart, SIGNAL(triggered(bool)), this, SLOT(onUnsetAutostart()));
+        m_actUnsetAutostart = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("media-playback-stop")), i18nc("Verb", "Remove Autostart"));
+        connect(m_actUnsetAutostart, &QAction::triggered, this, &SKGBookmarkPluginDockWidget::onUnsetAutostart);
 
-        m_actRename = m_mainMenu->addAction(KIcon("edit-rename"), i18nc("Verb, change the name of an item", "Rename"));
-        connect(m_actRename, SIGNAL(triggered(bool)), this, SLOT(onRenameBookmark()));
+        m_actRename = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("edit-rename")), i18nc("Verb, change the name of an item", "Rename"));
+        m_actRename->setShortcut(Qt::Key_F2);
+        connect(m_actRename, &QAction::triggered, this, &SKGBookmarkPluginDockWidget::onRenameBookmark);
 
-        m_actChangeIcon = m_mainMenu->addAction(KIcon("view-media-visualization"), i18nc("Verb, change the icon of an item", "Change icon..."));
-        connect(m_actChangeIcon, SIGNAL(triggered(bool)), this, SLOT(onChangeIconBookmark()));
+        m_actChangeIcon = m_mainMenu->addAction(SKGServices::fromTheme(QStringLiteral("edit-image-face-add")), i18nc("Verb, change the icon of an item", "Change icon..."));
+        connect(m_actChangeIcon, &QAction::triggered, this, &SKGBookmarkPluginDockWidget::onChangeIconBookmark);
 
         m_mainMenu->addSeparator();
-        m_mainMenu->addAction(SKGMainPanel::getMainPanel()->getGlobalAction("tab_overwritebookmark"));
+        m_mainMenu->addAction(SKGMainPanel::getMainPanel()->getGlobalAction(QStringLiteral("tab_overwritebookmark")));
 
         ui.kTools->setMenu(m_mainMenu);
     }
 }
 
-void SKGBookmarkPluginDockWidget::showMenu(const QPoint& iPos)
+void SKGBookmarkPluginDockWidget::showMenu(const QPoint iPos)
 {
     initMenu();
     m_mainMenu->popup(ui.kBookmarksList->mapToGlobal(iPos));
@@ -164,13 +164,13 @@ void SKGBookmarkPluginDockWidget::showMenu(const QPoint& iPos)
 void SKGBookmarkPluginDockWidget::onSetAutostart()
 {
     SKGTRACEINFUNC(10);
-    setAutostart("Y");
+    setAutostart(QStringLiteral("Y"));
 }
 
 void SKGBookmarkPluginDockWidget::onUnsetAutostart()
 {
     SKGTRACEINFUNC(10);
-    setAutostart("N");
+    setAutostart(QStringLiteral("N"));
 }
 
 void SKGBookmarkPluginDockWidget::setAutostart(const QString& value)
@@ -183,10 +183,10 @@ void SKGBookmarkPluginDockWidget::setAutostart(const QString& value)
     // foreach selected bookmark, set the t_autostart attribute to 'y' or 'n'
     {
         int nb = selectedBookmarks.count();
-        SKGBEGINPROGRESSTRANSACTION(*getDocument(), value == "Y" ? i18nc("Noun, name of the user action", "Autostart bookmarks") : i18nc("Noun, name of the user action", "Do not Autostart bookmarks"), err, nb);
+        SKGBEGINPROGRESSTRANSACTION(*getDocument(), value == QStringLiteral("Y") ? i18nc("Noun, name of the user action", "Autostart bookmarks") : i18nc("Noun, name of the user action", "Do not Autostart bookmarks"), err, nb);
         for (int i = 0 ; !err && i < nb  ; ++i) {
             SKGNodeObject bookmark(selectedBookmarks.at(i));
-            err = bookmark.setAttribute("t_autostart", value);
+            err = bookmark.setAttribute(QStringLiteral("t_autostart"), value);
             IFOKDO(err, bookmark.save())
 
             // Send message
@@ -197,7 +197,7 @@ void SKGBookmarkPluginDockWidget::setAutostart(const QString& value)
     }
 
     // status bar
-    IFOKDO(err, SKGError(0, value == "Y" ? i18nc("Successful message after an user action", "Autostart bookmarks") : i18nc("Successful message after an user action", "Do not Autostart bookmarks")))
+    IFOKDO(err, SKGError(0, value == QStringLiteral("Y") ? i18nc("Successful message after an user action", "Autostart bookmarks") : i18nc("Successful message after an user action", "Do not Autostart bookmarks")))
     SKGMainPanel::displayErrorMessage(err);
 }
 
@@ -207,7 +207,8 @@ void SKGBookmarkPluginDockWidget::openBookmark(const SKGNodeObject& iNode, bool
     SKGTRACEINFUNC(1);
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-    QList<SKGNodeObject> nodes;
+    QVector<SKGNodeObject> nodes;
+    nodes.reserve(20);
     nodes.push_back(iNode);
 
     int setForcusOn = 0;
@@ -245,7 +246,7 @@ void SKGBookmarkPluginDockWidget::openBookmark(const SKGNodeObject& iNode, bool
             // This bookmark is a folder ==> we open children by recursion
             SKGObjectBase::SKGListSKGObjectBase children;
             selectedNode.getNodes(children);
-            foreach(const SKGObjectBase & item, children) {
+            for (const auto& item : qAsConst(children)) {
                 nodes.push_back(SKGNodeObject(item));
             }
         }
@@ -264,12 +265,14 @@ void SKGBookmarkPluginDockWidget::onBeforeOpenBookmark()
 void SKGBookmarkPluginDockWidget::onOpenBookmark(const QModelIndex& index)
 {
     SKGTRACEINFUNC(1);
-    QSortFilterProxyModel* proxyModel = qobject_cast<QSortFilterProxyModel*>(ui.kBookmarksList->model());
-    SKGObjectModelBase* model = static_cast<SKGObjectModelBase*>(proxyModel->sourceModel());
-    if (model) {
-        SKGNodeObject node(model->getObject(proxyModel->mapToSource(index)));
-        if (!node.isFolder()) {
-            openBookmark(node);
+    if (!(QApplication::mouseButtons() & Qt::RightButton)) {
+        QSortFilterProxyModel* proxyModel = qobject_cast<QSortFilterProxyModel*>(ui.kBookmarksList->model());
+        SKGObjectModelBase* model = static_cast<SKGObjectModelBase*>(proxyModel->sourceModel());
+        if (model) {
+            SKGNodeObject node(model->getObject(proxyModel->mapToSource(index)));
+            if (!node.isFolder()) {
+                openBookmark(node);
+            }
         }
     }
 }
@@ -277,12 +280,14 @@ void SKGBookmarkPluginDockWidget::onOpenBookmark(const QModelIndex& index)
 void SKGBookmarkPluginDockWidget::onOpenBookmarkFolder(const QModelIndex& index)
 {
     SKGTRACEINFUNC(1);
-    QSortFilterProxyModel* proxyModel = qobject_cast<QSortFilterProxyModel*>(ui.kBookmarksList->model());
-    SKGObjectModelBase* model = static_cast<SKGObjectModelBase*>(proxyModel->sourceModel());
-    if (model) {
-        SKGNodeObject node(model->getObject(proxyModel->mapToSource(index)));
-        if (node.isFolder()) {
-            openBookmark(node);
+    if (!(QApplication::mouseButtons() & Qt::RightButton)) {
+        QSortFilterProxyModel* proxyModel = qobject_cast<QSortFilterProxyModel*>(ui.kBookmarksList->model());
+        SKGObjectModelBase* model = static_cast<SKGObjectModelBase*>(proxyModel->sourceModel());
+        if (model) {
+            SKGNodeObject node(model->getObject(proxyModel->mapToSource(index)));
+            if (node.isFolder()) {
+                openBookmark(node);
+            }
         }
     }
 }
@@ -296,7 +301,7 @@ void SKGBookmarkPluginDockWidget::onAddBookmarkGroup()
         // Get current selection name
         QString name;
         SKGObjectBase::SKGListSKGObjectBase bookmarks = getSelectedObjects();
-        if (bookmarks.count()) {
+        if (!bookmarks.isEmpty()) {
             SKGNodeObject parentNode(bookmarks.at(0));
             if (!parentNode.isFolder()) {
                 // This is not a folder, we have to take the parent
@@ -338,7 +343,7 @@ void SKGBookmarkPluginDockWidget::onAddBookmarks()
         // Get current selection name
         QString name;
         SKGObjectBase::SKGListSKGObjectBase bookmarks = getSelectedObjects();
-        if (bookmarks.count()) {
+        if (!bookmarks.isEmpty()) {
             SKGNodeObject parentNode(bookmarks.at(0));
             if (!parentNode.isFolder()) {
                 // This is not a folder, we have to take the parent
@@ -387,7 +392,7 @@ void SKGBookmarkPluginDockWidget::onAddBookmark()
         // Get current selection
         SKGNodeObject parentNode(getDocument(), 0);
         SKGObjectBase::SKGListSKGObjectBase bookmarks = getSelectedObjects();
-        if (bookmarks.count()) {
+        if (!bookmarks.isEmpty()) {
             parentNode = bookmarks.at(0);
         }
 
@@ -458,7 +463,6 @@ void SKGBookmarkPluginDockWidget::onRenameBookmark()
 {
     SKGTRACEINFUNC(1);
 
-    SKGObjectBase::SKGListSKGObjectBase selection;
     QItemSelectionModel* selModel = ui.kBookmarksList->selectionModel();
     QSortFilterProxyModel* proxyModel = qobject_cast< QSortFilterProxyModel* >(ui.kBookmarksList->model());
     if (proxyModel && selModel) {
@@ -480,6 +484,7 @@ void SKGBookmarkPluginDockWidget::onChangeIconBookmark()
     if (bookmarks.count() == 1) {
         SKGNodeObject node(bookmarks.at(0));
         KIconDialog diag(this);
+        diag.setup(KIconLoader::NoGroup);
         QString icon = diag.openDialog();
         if (!icon.isEmpty()) {
             SKGError err;
@@ -525,7 +530,7 @@ void SKGBookmarkPluginDockWidget::onBookmarkEditorChanged()
 
     // Enable bookmark buttons
     int nbSelectedObjects = getNbSelectedObjects();
-    bool testAdd = getDocument()->getDatabase() != NULL && nbSelectedObjects <= 1;
+    bool testAdd = getDocument()->getMainDatabase() != nullptr && nbSelectedObjects <= 1;
     if (m_actAddBookmarkGroup) {
         m_actAddBookmarkGroup->setEnabled(testAdd);
     }
@@ -555,10 +560,10 @@ void SKGBookmarkPluginDockWidget::refresh()
     if (nbSelectedObjects == 1) {
         SKGNodeObject node(bookmarks.at(0));
         if (m_actSetAutostart) {
-            m_actSetAutostart->setEnabled(node.getAttribute("t_autostart") != "Y");
+            m_actSetAutostart->setEnabled(node.getAttribute(QStringLiteral("t_autostart")) != QStringLiteral("Y"));
         }
         if (m_actUnsetAutostart) {
-            m_actUnsetAutostart->setEnabled(node.getAttribute("t_autostart") == "Y");
+            m_actUnsetAutostart->setEnabled(node.getAttribute(QStringLiteral("t_autostart")) == QStringLiteral("Y"));
         }
     } else {
         if (m_actSetAutostart) {
@@ -596,4 +601,4 @@ void SKGBookmarkPluginDockWidget::resizeEvent(QResizeEvent* iEvent)
     QWidget::resizeEvent(iEvent);
 }
 
-#include "skgbookmarkplugindockwidget.moc"
+
