@@ -24,6 +24,7 @@
 #include <klocalizedstring.h>
 #include <kio/filecopyjob.h>
 #include <kiconloader.h>
+#include <kaboutdata.h>
 #if QT_VERSION < 0x050700
 #include <KDELibs4Support/klocale.h>
 #endif
@@ -32,6 +33,7 @@
 #if QT_VERSION >= 0x050700
 #include <qlocale.h>
 #endif
+#include <qstandardpaths.h>
 #include <qsavefile.h>
 #include <qsqldatabase.h>
 #include <qsqlquery.h>
@@ -394,7 +396,7 @@ double SKGServices::stringToDouble(const QString& iNumber)
         }
     }
     if (Q_LIKELY(!ok)) {
-        SKGTRACE << "WARNING: SKGServices::stringToDouble(QStringLiteral(" << iNumber << ")) failed" << endl;
+        SKGTRACE << "WARNING: SKGServices::stringToDouble(" << iNumber << ") failed" << endl;
     }
     return output;
 }
@@ -406,6 +408,21 @@ QString SKGServices::doubleToString(double iNumber)
     return output;
 }
 
+QString SKGServices::getNextString(const QString& iString)
+{
+    QString output = iString;
+    bool ok;
+    qlonglong val = output.toLongLong(&ok);
+    if (Q_LIKELY(ok)) {
+        // This is a int
+        output = SKGServices::intToString(val + 1);
+    } else {
+        // This is a string
+        output = QLatin1String("");
+    }
+    return output;
+}
+
 QString SKGServices::dateToPeriod(const QDate iDate, const QString& iPeriod)
 {
     QString period;
@@ -1172,51 +1189,52 @@ SKGError SKGServices::cryptFile(const QString& iFileSource, const QString& iFile
 
     oModeSQLCipher = false;
 
-    QCA::Initializer init;
-    SKGTRACEL(10) << "QCA::Initializer done" << endl;
-    if (!iPassword.isEmpty() && !QCA::isSupported("aes128-ecb")) {
-        // Set error message
-        err.setReturnCode(ERR_INSTALL);  // To avoid password request
-        err.setMessage(i18nc("An error message about encryption", "AES128 encryption is not supported (%1). Please install qca-ossl.", QCA::supportedFeatures().join(QStringLiteral(","))));
-    } else {
-        // Create a random key - you'd probably use one from another
-        // source in a real application
-        QCA::SymmetricKey key(QByteArray("skrooge"));
-        QCA::Cipher* cipher = nullptr;
+    // Read document
+    QByteArray input;
+    QByteArray uba;
+    err = downloadToStream(QUrl::fromUserInput(iFileSource), input);
+    IFOK(err) {
+        bool isFileEncrypted = (input.startsWith(QByteArray((iHeaderFile % "_ENCRYPT").toLatin1())));
+        bool sqliteMode = (input.left(15) == "SQLite format 3");
+        SKGTRACEL(10) << "isFileEncrypted=[" << static_cast<unsigned int>(isFileEncrypted) << ']' << endl;
 
-        // Read document
-        QByteArray input;
-        QByteArray uba;
-        err = downloadToStream(QUrl::fromUserInput(iFileSource), input);
-        IFOK(err) {
-            bool isFileEncrypted = (input.startsWith(QByteArray((iHeaderFile % "_ENCRYPT").toLatin1())));
-            bool sqliteMode = (input.left(15) == "SQLite format 3");
-            SKGTRACEL(10) << "isFileEncrypted=[" << static_cast<unsigned int>(isFileEncrypted) << ']' << endl;
-
-            // !!!!! Remove Cipher encryption to remove security hole (thank you to Vincent P) !!!!!
-            // Only in sqlcipher mode. WARNING: in sqlite mode the issue is still there => add a message to push people to swith in sqlcipher mode
-            if (iEncrypt && !sqliteMode) {
-                // The input file is a sqlcipher file and we must save it
-                // We just have to add a new header to the input file
-                uba.reserve(input.length() + iHeaderFile.length() + 11);
-                uba.append(iHeaderFile.toLatin1());
-                uba.append(!iPassword.isEmpty() ? "_ENCRYPTE3-" : "_DECRYPTE3-");
-                uba.append(input);
-                oModeSQLCipher = true;
-            } else if (!iEncrypt && input.startsWith(QByteArray((iHeaderFile % "_ENCRYPTE3-").toLatin1()))) {
-                // This check is done to improve performances
-                if (iPassword.isEmpty() || iPassword == QStringLiteral("DEFAULTPASSWORD")) {
-                    err = SKGError(ERR_ENCRYPTION, i18nc("Error message", "Wrong password"));
-                } else {
-                    // The input file encrypter with the new mode
-                    // We just have to remove the header
-                    if (!iHeaderFile.isEmpty() && input.startsWith(iHeaderFile.toLatin1())) {
-                        input = input.right(input.length() - iHeaderFile.length() - 11);
-                    }
-                    uba = input;
-                    oModeSQLCipher = true;
-                }
+        // !!!!! Remove Cipher encryption to remove security hole (thank you to Vincent P) !!!!!
+        // Only in sqlcipher mode. WARNING: in sqlite mode the issue is still there => add a message to push people to swith in sqlcipher mode
+        if (iEncrypt && !sqliteMode) {
+            // The input file is a sqlcipher file and we must save it
+            // We just have to add a new header to the input file
+            uba.reserve(input.length() + iHeaderFile.length() + 11);
+            uba.append(iHeaderFile.toLatin1());
+            uba.append(!iPassword.isEmpty() ? "_ENCRYPTE3-" : "_DECRYPTE3-");
+            uba.append(input);
+            oModeSQLCipher = true;
+        } else if (!iEncrypt && input.startsWith(QByteArray((iHeaderFile % "_ENCRYPTE3-").toLatin1()))) {
+            // This check is done to improve performances
+            if (iPassword.isEmpty() || iPassword == QStringLiteral("DEFAULTPASSWORD")) {
+                err = SKGError(ERR_ENCRYPTION, i18nc("Error message", "Wrong password"));
             } else {
+                // The input file encrypter with the new mode
+                // We just have to remove the header
+                if (!iHeaderFile.isEmpty() && input.startsWith(iHeaderFile.toLatin1())) {
+                    input = input.right(input.length() - iHeaderFile.length() - 11);
+                }
+                uba = input;
+                oModeSQLCipher = true;
+            }
+        } else {
+            // WARNING: This part is not really secured but is kept for compatibility
+            SKGTRACEL(10) << "Mode not secured" << endl;
+            QCA::Initializer init;
+            QCA::SymmetricKey key(QByteArray("skrooge"));
+
+            SKGTRACEL(10) << "QCA::Initializer done" << endl;
+            if (!iPassword.isEmpty() && !QCA::isSupported("aes128-ecb")) {
+                // Set error message
+                err.setReturnCode(ERR_INSTALL);  // To avoid password request
+                err.setMessage(i18nc("An error message about encryption", "AES128 encryption is not supported (%1). Please install qca-ossl.", QCA::supportedFeatures().join(QStringLiteral(","))));
+            } else {
+                QCA::Cipher* cipher = nullptr;
+
                 QCA::InitializationVector iv(iPassword.toLatin1());
 
                 // Create a 128 bit AES cipher object using Cipher Block Chaining (CBC) mode
@@ -1284,31 +1302,31 @@ SKGError SKGServices::cryptFile(const QString& iFileSource, const QString& iFile
                         uba = uba.insert(0, h);
                     }
                 }
+
+                delete cipher;
+                cipher = nullptr;
             }
-            SKGTRACEL(10) << "output 2=[" << uba.left(50) << "...]" << endl;
+        }
+        SKGTRACEL(10) << "output 2=[" << uba.left(50) << "...]" << endl;
 
-            // output the results of that stage
-            IFOK(err) {
-                SKGTRACEIN(10, "SKGServices::cryptFile-save file");
-                QSaveFile fileOutput(iFileTarget);
-                if (!fileOutput.open(QIODevice::WriteOnly)) {
-                    err = SKGError(ERR_WRITEACCESS, i18nc("Error message: writing a file failed", "Write file '%1' failed", iFileTarget));
-                } else {
-                    // Write document
-                    fileOutput.write(uba);
+        // output the results of that stage
+        IFOK(err) {
+            SKGTRACEIN(10, "SKGServices::cryptFile-save file");
+            QSaveFile fileOutput(iFileTarget);
+            if (!fileOutput.open(QIODevice::WriteOnly)) {
+                err = SKGError(ERR_WRITEACCESS, i18nc("Error message: writing a file failed", "Write file '%1' failed", iFileTarget));
+            } else {
+                // Write document
+                fileOutput.write(uba);
 
-                    // Close the file
-                    if (!fileOutput.commit()) {
-                        IFOK(err) {
-                            err = SKGError(ERR_WRITEACCESS, i18nc("Error message: writing a file failed", "Write file '%1' failed", iFileTarget));
-                        }
+                // Close the file
+                if (!fileOutput.commit()) {
+                    IFOK(err) {
+                        err = SKGError(ERR_WRITEACCESS, i18nc("Error message: writing a file failed", "Write file '%1' failed", iFileTarget));
                     }
                 }
             }
         }
-
-        delete cipher;
-        cipher = nullptr;
     }
     SKGTRACEL(10) << "Output parameter [oModeSQLCipher]=[" << static_cast<unsigned int>(oModeSQLCipher) << ']' << endl;
     return err;
@@ -1867,8 +1885,108 @@ QIcon SKGServices::fromTheme(const QString& iName, const QStringList& iOverlays)
         output = KDE::icon(iName);
     }
     if (output.isNull() && !iName.isEmpty()) {
-        SKGTRACE << "WARNING: Icon [" << iName << "] not found" << endl;
-        output = KDE::icon(QStringLiteral("script-error"));
+        static QHash<QString, QString> alternatives;
+        if (alternatives.count() == 0) {
+            // Build alternatives
+            alternatives[QStringLiteral("arrow-down")] = QStringLiteral("go-down");
+            alternatives[QStringLiteral("arrow-right")] = QStringLiteral("go-next");
+            alternatives[QStringLiteral("arrow-up")] = QStringLiteral("go-up");
+            alternatives[QStringLiteral("arrow-down-double")] = QStringLiteral("go-down");
+            alternatives[QStringLiteral("arrow-up-double")] = QStringLiteral("go-up");
+            alternatives[QStringLiteral("bookmark")] = QStringLiteral("bookmark-new");
+            alternatives[QStringLiteral("bookmarks")] = QStringLiteral("bookmark-new");
+            alternatives[QStringLiteral("checkbox")] = QStringLiteral("emblem-symbolic-link");
+            alternatives[QStringLiteral("chronometer")] = QStringLiteral("appointment");
+            alternatives[QStringLiteral("configure")] = QStringLiteral("preferences-desktop");
+            alternatives[QStringLiteral("dashboard-show")] = QStringLiteral("user-desktop");
+            alternatives[QStringLiteral("dialog-cancel")] = QStringLiteral("process-stop");
+            alternatives[QStringLiteral("dialog-close")] = QStringLiteral("process-stop");
+            alternatives[QStringLiteral("dialog-ok")] = QStringLiteral("");
+            alternatives[QStringLiteral("download-later")] = QStringLiteral("applications-internet");
+            alternatives[QStringLiteral("download")] = QStringLiteral("applications-internet");
+            alternatives[QStringLiteral("draw-freehand")] = QStringLiteral("accessories-text-editor");
+            alternatives[QStringLiteral("edit-guides")] = QStringLiteral("text-x-generic");
+            alternatives[QStringLiteral("edit-rename")] = QStringLiteral("accessories-text-editor");
+            alternatives[QStringLiteral("emblem-locked")] = QStringLiteral("lock");
+            alternatives[QStringLiteral("exchange-positions")] = QStringLiteral("");
+            alternatives[QStringLiteral("format-fill-color")] = QStringLiteral("");
+            alternatives[QStringLiteral("games-solve")] = QStringLiteral("application-certificate");
+            alternatives[QStringLiteral("get-hot-new-stuff")] = QStringLiteral("applications-other");
+            alternatives[QStringLiteral("irc-operator")] = QStringLiteral("");
+            alternatives[QStringLiteral("ktip")] = QStringLiteral("dialog-information");
+            alternatives[QStringLiteral("labplot-xy-plot-two-axes-centered-origin")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("layer-visible-off")] = QStringLiteral("");
+            alternatives[QStringLiteral("layer-visible-on")] = QStringLiteral("");
+            alternatives[QStringLiteral("merge")] = QStringLiteral("");
+            alternatives[QStringLiteral("office-chart-area")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("office-chart-area-stacked")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("office-chart-bar-percentage")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("office-chart-bar")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("office-chart-bar-stacked")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("office-chart-line")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("office-chart-line-stacked")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("office-chart-pie")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("office-chart-ring")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("map-flat")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("office-chart-scatter")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("preview")] = QStringLiteral("document-print-preview");
+            alternatives[QStringLiteral("quickopen")] = QStringLiteral("emblem-symbolic-link");
+            alternatives[QStringLiteral("run-build-configure")] = QStringLiteral("media-playback-start");
+            alternatives[QStringLiteral("run-build")] = QStringLiteral("media-playback-start");
+            alternatives[QStringLiteral("show-menu")] = QStringLiteral("applications-system");
+            alternatives[QStringLiteral("skrooge_category")] = QStringLiteral("folder-open");
+            alternatives[QStringLiteral("split")] = QStringLiteral("edit-cut");
+            alternatives[QStringLiteral("taxes-finances")] = QStringLiteral("fonts");
+            alternatives[QStringLiteral("tools-wizard")] = QStringLiteral("applications-other");
+            alternatives[QStringLiteral("user-group-properties")] = QStringLiteral("system-users");
+            alternatives[QStringLiteral("user-properties")] = QStringLiteral("document-properties");
+            alternatives[QStringLiteral("utilities-file-archiver")] = QStringLiteral("package-x-generic");
+            alternatives[QStringLiteral("vcs-conflicting")] = QStringLiteral("dialog-warning");
+            alternatives[QStringLiteral("vcs-normal")] = QStringLiteral("dialog-information");
+            alternatives[QStringLiteral("view-bank-account-checking")] = QStringLiteral("go-home");
+            alternatives[QStringLiteral("view-bank-account")] = QStringLiteral("x-office-address-book");
+            alternatives[QStringLiteral("view-bank-account-savings")] = QStringLiteral("go-home");
+            alternatives[QStringLiteral("view-bank")] = QStringLiteral("go-home");
+            alternatives[QStringLiteral("view-calendar-journal")] = QStringLiteral("x-office-calendar");
+            alternatives[QStringLiteral("view-calendar-month")] = QStringLiteral("x-office-calendar");
+            alternatives[QStringLiteral("view-calendar")] = QStringLiteral("x-office-calendar");
+            alternatives[QStringLiteral("view-calendar-week")] = QStringLiteral("x-office-calendar");
+            alternatives[QStringLiteral("view-calendar-whatsnext")] = QStringLiteral("x-office-calendar");
+            alternatives[QStringLiteral("view-categories")] = QStringLiteral("folder-open");
+            alternatives[QStringLiteral("view-categories-expenditures")] = QStringLiteral("face-sad");
+            alternatives[QStringLiteral("view-categories-incomes")] = QStringLiteral("face-smile");
+            alternatives[QStringLiteral("view-file-columns")] = QStringLiteral("go-home");
+            alternatives[QStringLiteral("view-financial-list")] = QStringLiteral("go-home");
+            alternatives[QStringLiteral("view-investment")] = QStringLiteral("go-home");
+            alternatives[QStringLiteral("view-list-details")] = QStringLiteral("go-home");
+            alternatives[QStringLiteral("view-list-text")] = QStringLiteral("go-home");
+            alternatives[QStringLiteral("view-pim-calendar")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("view-statistics")] = QStringLiteral("x-office-spreadsheet");
+            alternatives[QStringLiteral("window-duplicate")] = QStringLiteral("edit-copy");
+            alternatives[QStringLiteral("zoom-fit-width")] = QStringLiteral("media-playback-stop");
+            alternatives[QStringLiteral("smallclock")] = QStringLiteral("");
+            alternatives[QStringLiteral("edit_undo")] = QStringLiteral("edit-undo");
+        }
+        bool alternativeEmpty = false;
+        if (alternatives.contains(iName)) {
+            auto alternative = alternatives.value(iName);
+            alternativeEmpty = (alternative.isEmpty());
+            if (!alternativeEmpty) {
+                if (!iOverlays.isEmpty()) {
+                    output = KDE::icon(alternative, iOverlays);
+                } else {
+                    output = KDE::icon(alternative);
+                }
+            }
+        }
+        if (output.isNull() && !alternativeEmpty) {
+            SKGTRACE << "WARNING: Icon [" << iName << "] not found" << endl;
+            output = KDE::icon(QStringLiteral("script-error"));
+            if (output.isNull()) {
+                output = KDE::icon(QStringLiteral("image-missing"));
+            }
+        }
+
     }
     return output;
 }
@@ -1885,3 +2003,17 @@ QString SKGServices::getMajorVersion(const QString& iVersion)
     }
     return output;
 }
+
+QString SKGServices::getFullPathCommandLine(const QString& iCommandLine)
+{
+    QString output = iCommandLine;
+    if (!output.isEmpty()) {
+        auto pathWords = SKGServices::splitCSVLine(output, QLatin1Char(' '));
+        QString fullpath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, KAboutData::applicationData().componentName() % "/" % pathWords.at(0));
+        if (!fullpath.isEmpty()) {
+            pathWords[0] = fullpath;
+            output = pathWords.join(QLatin1Char(' '));
+        }
+    }
+    return output;
+}
